;  AntiISDA Warrior     (Sega Master System Version) by
; Ventzislav Tzvetkov  http://drhirudo.hit.bg Copyright  2003-2004
;Based on the Vic-20  version, which is based on the GameBoy  version, which is
;based on the IBM-PC  version, which is based on the Amiga    version, which is
;based on the Apple   version, which is based on the Oric     version, which is
;based on the Sinclair version.


;History:
;Version 1.02   - major bugfixes. Now works correctly with MekaW.
;Version 1.0101 - minor bugfixes, thanks to Bock for suggesting them.

.MEMORYMAP
SLOTSIZE $4000
DEFAULTSLOT 0
SLOT 0 $0000
SLOT 1 $4000
.ENDME

.ROMBANKMAP
BANKSTOTAL 2
BANKSIZE $4000
BANKS 2
.ENDRO

.EMPTYFILL $C9                ;ret.
.COMPUTESMSCHECKSUM           ;compute sms checksum for this rom.

.define Level  $C000
.define seed1  $C001
.define seed2  $C002
.define prevr  $C003
.define count  $C004
.define YPosition $C005
.define XPosition $C006
.define DeltaX    $C007
.define Ammo      $C008
.define Pause     $C009
;
; standard stuff?
;

.BANK 0 SLOT 0
.ORG 0

;
; main
;
    di          ;Disable interrupts.
    im 1        ;Interrupt mode 1.
    ld sp, $DFF0 ;Set stack pointer.

    jp main        ;Jump to the main part of the program.
.db "drHirudo"
vdp_reg:

; Register 0

.db %00000100
.db %10000000

; Register 1

.db %10000000
.db %10000001

; Register 2

.db %11111111
.db %10000010

; Register 5

.db %11111111
.db %10000101

; Register 6

.db %11111011
.db %10000110

; Register 7

.db %00000010
.db %10000111

; Register 8

.db %00000000
.db %10001000

; Register 9

.db %00000000
.db %10001001

; Register 10

.db %00000000
.db %10001010

vdp_registers_end:
vdp_size: .dw vdp_registers_end-vdp_reg

.org $0028           ;WaitTOF(b); subroutine called via rst $28
WaitVBL:             ;b contains the number of vertical blanks to wait.
        in a,($7E)   ;Wait for vertical Blank.
        cp $B0
        jr nz, WaitVBL
        nop
        nop
        nop
        djnz WaitVBL

.org $0038            ;HideSprite(); Subroutine
HideSprite:           ;Hides the sprite by setting $D0 as Y position.
        sub a         ;A=0
        out ($BF), a        ; to the VDP control port.
        ld a, $7f           ; at $7F00 begins my sprite table.
        out ($BF), a
        ld a,$D0
        out ($BE),a
        ret

.org $0066         ;Pause handler. Removed because it freezed the machine.
;    ld a,(Pause)
;    cp 0
;    jr z,NoPause
;    sub a
;    ld (Pause),a
    retn
;NoPause:
;    inc a
;    ld (Pause),a
;    jr NoPause

    ; * which starts right here.

main:

    ld hl, vdp_reg

    ld c, $BF

    ld b, vdp_size

    otir

;Clear VRAM

    sub a    ; We output two bytes to the VDP control port
    out ($BF), a    ; to indicate that we want to write to VRAM
    ld a, %01000000    ; and to set the VRAM address pointer to
    out ($BF), a    ; $0000.

    ld bc, $4000    ; The VRAM is 16 Kilobytes in size.

    sub a
vram_clear_loop:
    out ($BE), a
    dec c
    jr nz, vram_clear_loop
    dec b
    jr nz, vram_clear_loop

    ;Write to CRAM to set the palette.

    ld hl, palette    ; load HL with start address of palette
            ; data.
    call SetPalette

tile_init:

    ld a, $00
    out ($BF), a
     ld a, $40
     out ($BF), a
    ld hl, Tiles ; Load HL with tiles address.

     ld bc, Tiles_size 

tile_loop:

     ld a, (HL)

     out ($BE), a
     inc hl
     dec c
     jr nz, tile_loop
     dec b
     jr nz, tile_loop

.define Screen_address $7800      ;Screen memory+$40(hi)

TitleScreen:
    sub a       ;clear a
    out ($BF),a

     ld a, $78
                
     out ($BF), a        ; Then we output this byte to the VDP
                ; control port.
    sub a                   ; Clear a;

Title_loop:
    inc a            ;Set tiles from 0 to $60 - they contain the Title Logo.
    out ($BE), a     ; Output tile number.
    push af
    ld  a,%00000000  ;Set 0 in the Tile data byte.
    out ($BE), a
    pop af
    cp $60
    jr nz, Title_loop
    sub a
    ld b,130    ;2 rows (32*2*2) + 1 tile of empty space under the Logo.
TL1:    out ($BE),a
        dec b
        jr nz, TL1

    ld hl, TitleText

    call PrintText

    rst $38   ;Hide Sprite.

  ld hl, PSGMOD_SONG
  call PSGMOD_INITIATE_NEW_SONG

;Turn on the VDP

     ld a, %11000000         ; set bits 6 and 7
     out ($BF), a
     ld a, %10000001         ; indicates a write to VDP reg. 1
     out ($BF), a

;Wait for button press.

TitleLoop:
  in a,($7E)   ;Wait for vertical Blank.
  cp $6D
  jr nz, TitleLoop
  ld hl,palette2
  call SetPalette
TitleLoop2:
  in a,($7E)   ;Wait for vertical Blank.
  cp $9D
  jr nz, TitleLoop2
  ld hl,palette
  call SetPalette
  call PSGMOD_PLAY_NEXT_NOTE      ; play next note
     inc b
     in a,($DC)
     bit 4,a                 ;Check if A button was pressed.
     jr z,GBegin             ;Yes - exit from the loop.
     bit 5,a                 ;Check if B button was pressed.
     jr nz,TitleLoop

GBegin:
     call psg_init
     ld a,1
     ld (Level),a
Initrand:
     ld a,b
     ld (seed1),a
     ld (seed2),a

GLoop:
  in a,($7E)   ;Wait for vertical Blank.
  cp $B0
  jr nz, GLoop
       ld hl, BlackPalette ;Load HL with start address of palette data.
       call SetPalette
       call vdp_clear
       ld a,(Level)
       cp 8
       jp z,Success
       add a,a
       ld b,a
       ld a,26              ;Ammo = 26-Level*2
       sub b
       ld (Ammo),a

       ld bc,Screen_address+64*10+24    ; load BC Screen_address+offset.
       ld a, c                  ;This could be optimized.
       out ($BF), a

       ld a, b
       out ($BF), a
       ld hl,StageText
       call PrintText
       ld a,(Level)
       add a,176
       out ($BE),a
       rst $38
       ld hl,palette2
       call SetPalette

       ld b,255              ;255 vertical blanks ~ 5 secs.

       rst $28               ;Wait b VBLs
       call vdp_clear
       ld a,b
       ld (count),a

rnd:
        ld a,(seed2)     ;Calculate pseudo random number. Adapted from 6502
        inc a            ;routine, which explains the mess.
        ld (seed2),a
        ld l,a
        ld a,(seed1)
        add a,l
        adc a,a
        ld (seed1),a
        xor l
        ld e,l
        ld (seed2),a
        ld a,(prevr)
        ld l,a
        ld a,e
        sub l
        ld (prevr),a
        and 31
        add a,a
        ld e,a
        ld a,(Level)
        ld d,a
        ld hl,Screen_address-$4000
        ld bc,64
Offset: add hl,bc    ;Gets agents screen position, according to the level.
     dec d
     jr nz,Offset
     add hl,de
     ld a, l            ; Writes the low byte of the VRAM address
     out ($BF), a       ; to the VDP control port.
     ld a, h
     out ($BF), a       ; Then we output this byte to the VDP
     in  a,($BE)
     cp 0
     jr nz,rnd
     ld de,$4000
     add hl,de
     ld a, l            ; Writes the low byte of the VRAM address
     out ($BF), a       ; to the VDP control port.
     ld a, h
     out ($BF), a       ; Then we output this byte to the VDP
     ld b,149
     ld a,(count)
     add a,b
     out ($BE),a

     ld a,(count)
     inc a
     ld (count),a
     cp 10
     jp nz,rnd

     ld hl,palette
     call SetPalette
     sub a
     ld (Pause),a
     jp Bomb

Exit:
     call vdp_clear
     jp TitleScreen

Success:
     call vdp_clear
     rst $38
     ld bc,Screen_address+16
     ld a, c
     out ($BF), a
     ld a, b
     out ($BF), a
     ld hl,CongratulationsText
     call PrintText
  ld hl, PSGMOD_SONG3
  call PSGMOD_INITIATE_NEW_SONG

     ld d,50
     ld c,10
SuccessScreen:
  ld hl,palette4
  call SetPalette
SuccessScreenUp:
  in a,($7E)   ;Wait for vertical Blank.
  cp $0D
  jr nz, SuccessScreenUp
  ld hl,palette2
  call SetPalette
SuccessScreenMiddle:
  in a,($7E)   ;Wait for vertical Blank.
  cp $75
  jr nz, SuccessScreenMiddle
  ld hl,palette
  call SetPalette
SuccessScreen2:
  in a,($7E)   ;Wait for vertical Blank.
  cp $A0
  jr nz, SuccessScreen2
  ld hl,palette2
  call SetPalette
  ld a,%00000100
  ld ($FFFD),a
  call PSGMOD_PLAY_NEXT_NOTE      ; play next note
     in a,($DC)
     bit 4,a                 ;Check if A button was pressed.
     jr z,SuccessEnd         ;Yes - exit from the loop.
     bit 5,a                 ;Check if B button was pressed.
     jr z,SuccessEnd
     dec d
     jr nz, SuccessScreen
     dec c
     jr nz, SuccessScreen
SuccessEnd:
     call psg_init
     call vdp_clear
     ld b,50         ;50 vertical blanks.
     rst $28         ;Wait for vertical Blank.
     ld hl,BlackPalette
     call SetPalette
     jp TitleScreen

palette:
.include "palette.s"

Tiles:
.incbin "tiles.bin"
Tiles_end:
Tiles_size: .dw Tiles_end-Tiles

TitleText: .db 'y','o'+26,'u'+26,'r'+26,0,'s'+26,'e'+26,'r'+26,'v'+26,'e'+26,'r'+26,0,'f'+26,'u'+26,'l'+26,'l'+26,0
.db 'w'+26,'i'+26,'t'+26,'h'+26,0,'a'+26,'l'+26,'l'+26,0,'t'+26,'h'+26,'o'+26,'s'+26,'e'+26,'f'+26,'i'+26,'f'+26,'t'+26,'e'+26,'e'+26,'n'+26
.db 0,'y'+26,'e'+26,'a'+26,'r'+26,'s'+26,0,0,'o'+26,'l'+26,'d'+26,0,0,'s','e'+26,'g'+26,'a'+26,0,0,'m','a'+26,'s'+26,'t'+26,'e'+26,'r'+26
.db 's','y'+26,'s'+26,'t'+26,'e'+26,'m'+26,0,'w'+26,'a'+26,'r'+26,'e'+26,'z'+26,160,0,0,'o'+26,'f'+26,0,0,'w'+26,'h'+26,'i'+26,'c'+26,'h'+26,0,0
.db 'n'+26,'o'+26,'b'+26,'o'+26,'d'+26,'y'+26,'c'+26,'a'+26,'r'+26,'e'+26,'s'+26,0,'a'+26,'n'+26,'y'+26,'m'+26,'o'+26,'r'+26,'e'+26,0
.db 's'+26,'i'+26,'n'+26,'c'+26,'e'+26,0,'m'+26,'i'+26,'l'+26,'l'+26,'i'+26,'o'+26,'n'+26,0,'a'+26,'g'+26,'e'+26,'s'+26,
.db 'u'+26,'n'+26,'f'+26,'o'+26,'r'+26,'t'+26,'u'+26,'n'+26,'a'+26,'t'+26,'e'+26,'l'+26,'y'+26,0,0,'i'+26,'s'+26,0
.db 'b'+26,'e'+26,'i'+26,'n'+26,'g'+26,0,'a'+26,'t'+26,'t'+26,'a'+26,'c'+26,'k'+26,'e'+26,'d'+26,'b'+26,'y'+26,0,'s'+26,'o'+26,'m'+26,'e'+26,0,
.db 0,162,163,164,165,0,0,'a'+26,'g'+26,'e'+26,'n'+26,'t'+26,'s'+26,160,0,0,'a'+26,'n'+26,'d'+26,0,'t'+26,'h'+26,'e'+26,'y'+26,'w'+26,'a'+26,'n'+26,'t'+26,0
.db 't'+26,'o'+26,0,'f'+26,'o'+26,'r'+26,'m'+26,'a'+26,'t'+26,0,0,'y'+26,'o'+26,'u'+26,'r'+26,0,0,'o'+26,'n'+26,'l'+26,'y'+26,0,0,'h'+26,'a'+26,'r'+26,'d'+26
.db 'd'+26,'i'+26,'s'+26,'k'+26,0,'y'+26,'o'+26,'u'+26,0,'e'+26,'v'+26,'e'+26,'r'+26,0,'o'+26,'w'+26,'n'+26,'e'+26,'d'+26,0,166,168,169,'m','b',167,161
.dsb 38,0
.db 't','h'+26,'e'+26,'y'+26,0,'p'+26,'e'+26,'n'+26,'e'+26,'t'+26,'r'+26,'a'+26,'t'+26,'e'+26,0,0,'f'+26,'r'+26,'o'+26,'m'+26,0,0
.db 'd'+26,'i'+26,'f'+26,'f'+26,'e'+26,'r'+26,'e'+26,'n'+26,'t'+26,'p'+26,'o'+26,'r'+26,'t'+26,'s'+26,160,0,'s'+26,'o'+26,0,'y'+26,'o'+26,'u'+26,0
.db 0,'h'+26,'a'+26,'v'+26,'e'+26,0,'t'+26,'o'+26,0,'a'+26,'i'+26,'m'+26,0,0,'y'+26,'o'+26,'u'+26,'r'+26,'p'+26,'i'+26,'n'+26,'g'+26,0
.db 'b'+26,'o'+26,'m'+26,'b'+26,'s'+26,0,'m'+26,'o'+26,'r'+26,'e'+26,0,0,'p'+26,'r'+26,'e'+26,'c'+26,'i'+26,'s'+26,'e'+26,'l'+26,'y'+26,0,0,'w'+26,'i'+26,'t'+26,'h'+26
.db 't'+26,'h'+26,'e'+26,0,'j'+26,'o'+26,'y'+26,'p'+26,'a'+26,'d'+26,161,0,'p','r'+26,'e'+26,'s'+26,'s'+26,0,212,0,'t'+26,'o'+26,0,'s'+26,'h'+26,'o'+26,'o'+26,'t'+26,161
BlackPalette: .dsb 67,0
.db 166,'c',167,0,178,176,176,180,0,0,'b'+26,'y'+26,0,'v','e'+26,'n'+26,'t'+26,'z'+26,'i'+26,'s'+26,'l'+26,'a'+26,'v'+26,0,'t','z'+26,'v'+26,'e'+26,'t'+26,'k'+26,'o'+26,'v'+26
.dsb 64,0
.db 188,188,188,188,189,190,191,192,193,194,195,195,196,197,198,199,200,201,202,203,204,205,206,207,208,209,210,211,188,188,188,188
.db $FF
StageText: .db 's','t'+26,'a'+26,'g'+26,'e'+26,0,$FF
CongratulationsText: .db "congratulations",187
.dsb 52,0
.db "you",0,"won",187
.dsb 76,0
.db "a",'l'+26,'l'+26,0,'t'+26,'h'+26,'e'+26,0,162,163,164,165,0,'a'+26,'g'+26,'e'+26,'n'+26,'t'+26,'s'+26,0,'a'+26,'r'+26,'e'+26,0,'g'+26,'o'+26,'n'+26,'e'+26,0,'a'+26,'n'+26,'d'+26
.db 'y'+26,'o'+26,'u'+26,0,'j'+26,'u'+26,'s'+26,'t'+26,0
.db 'f'+26,'i'+26,'n'+26,'d'+26,0,'t'+26,'h'+26,'a'+26,'t'+26,0,'o'+26,'n'+26,'e'+26,0,0,'o'+26,'f'+26,0,'t'+26,'h'+26,'e'+26,'s'+26,'e'+26,
.db 'j'+26,'e'+26,'r'+26,'k'+26,'s'+26,0,'h'+26,'o'+26,'l'+26,'d'+26,'e'+26,'d'+26,0,'a'+26,0,0,'p'+26,'i'+26,'r'+26,'a'+26,'t'+26,'e'+26,'d'+26,0,0,'c'+26,'o'+26,'p'+26,'y'+26,0,'o'+26,'f'+26
.db 186,'s','t'+26,'a'+26,'c'+26,'k'+26,'e'+26,'r'+26,186,160,0,'w'+26,'h'+26,'i'+26,'c'+26,'h'+26,0,'d'+26,'o'+26,'u'+26,'b'+26,'l'+26,'e'+26,'s'+26,0,'u'+26,'p'+26,0,'y'+26,'o'+26,'u'+26,'r'+26
.db 'd'+26,'i'+26,'s'+26,'k'+26,'s'+26,0,'s'+26,'p'+26,'a'+26,'c'+26,'e'+26,0,'a'+26,'s'+26,0,'t'+26,'h'+26,'e'+26,0,'r'+26,'e'+26,'a'+26,'d'+26,'m'+26,'e'+26,0,0,'s'+26,'a'+26,'y'+26,'s'+26,161
.db 'n','o'+26,'w'+26,0,'y'+26,'o'+26,'u'+26,0,0,'c'+26,'a'+26,'n'+26,0,0,'u'+26,'p'+26,'l'+26,'o'+26,'a'+26,'d'+26,0,0,'e'+26,'v'+26,'e'+26,'n'+26,0,0,'m'+26,'o'+26,'r'+26,'e'+26,
.db 'g'+26,'r'+26,'e'+26,'a'+26,'t'+26,0,'b'+26,'i'+26,'g'+26,0,'s'+26,'o'+26,'f'+26,'t'+26,0,'a'+26,'n'+26,'d'+26,0,'h'+26,'a'+26,'r'+26,'d'+26,'w'+26,'a'+26,'r'+26,'e'+26,'z'+26,161,0,0,0
.db 't','h'+26,'a'+26,'n'+26,'k'+26,'s'+26,0,'t'+26,'o'+26,0,162,163,164,165,161
.dsb 153,0
.db 't','h'+26,'a'+26,'n'+26,'k'+26,'s'+26,0,'f'+26,'o'+26,'r'+26,0,'p'+26,'l'+26,'a'+26,'y'+26,'i'+26,'n'+26,'g'+26,187
.db $FF
ShootsText: .db 170,171,172,172,173,174,175,$FF
GameOverText: .db 'h','a'+26,'r'+26,'d'+26,0,'l'+26,'u'+26,'c'+26,'k'+26,160,0,'w'+26,'e'+26,'b'+26,'m'+26,'a'+26,'s'+26,'t'+26,'e'+26,'r'+26,187
.dsb 38,0
.db 'a','l'+26,'l'+26,0,'y'+26,'o'+26,'u'+26,'r'+26,0,0,'a'+26,'m'+26,'m'+26,'u'+26,'n'+26,'i'+26,'t'+26,'i'+26,'o'+26,'n'+26,'s'+26,0,0,'a'+26,'r'+26,'e'+26,0,'g'+26,'o'+26,'n'+26,'e'+26,'a'+26,'n'+26,'d'+26,0,'y'+26,'o'+26,'u'+26,0
.db 'j'+26,'u'+26,'s'+26,'t'+26,0,'c'+26,'a'+26,'n'+26,0,'n'+26,'o'+26,'t'+26,0,'h'+26,'o'+26,'l'+26,'d'+26,0,'i'+26,'t'+26,0,'a'+26,'n'+26,'y'+26,'m'+26,'o'+26,'r'+26,'e'+26,161,0
.db 'y','o'+26,'u'+26,'r'+26,0,0,'s'+26,'i'+26,'t'+26,'e'+26,0,0,'w'+26,'a'+26,'s'+26,0,'c'+26,'l'+26,'o'+26,'s'+26,'e'+26,'d'+26,0,'a'+26,'n'+26,'d'+26
.db 'y'+26,'o'+26,'u'+26,'r'+26,0,'h'+26,'a'+26,'r'+26,'d'+26,0,0,'d'+26,'i'+26,'s'+26,'k'+26,0,'i'+26,'s'+26,0,'n'+26,'o'+26,'w'+26,0,0,'l'+26,'o'+26,'w'+26
.db 'l'+26,'e'+26,'v'+26,'e'+26,'l'+26,'f'+26,'o'+26,'r'+26,'m'+26,'a'+26,'t'+26,'e'+26,'d'+26,160,0,'p'+26,'l'+26,'u'+26,'s'+26,0,0,'b'+26,'r'+26,'o'+26,'k'+26,'e'+26,'n'+26,0,'b'+26,'y'+26,0,0,'t'+26,'h'+26,'e'+26,'s'+26,'e'+26
.db 'f'+26,'e'+26,'l'+26,'l'+26,'o'+26,'w'+26,0,'c'+26,'h'+26,'a'+26,'p'+26,'s'+26,161
.db $FF
GameOverText2: .db "game",0,0,"over",187,$FF

;Subroutines

             ;Print Texts at *hl, at the current position, $FF marks end.
PrintText:
    ld a, (hl)        ; Load byte of message into accumulator.
    cp $FF            ; A $FF byte indicates end of text.
    jr z, EndOfText

    out ($BE), a
    sub a             ;A=0
    out ($BE), a      ;No tile configs.


    inc hl
    jr PrintText
                
EndOfText:
        ret

SetPalette:

    ld b, $20

    ld c, $BE
     
    sub a
    out ($BF), a
    ld a, $C0
    out ($BF), a
    otir
    ret

vdp_clear:
    ld bc,Screen_address
    ld a, c
    out ($BF), a
    ld a, b
    out ($BF), a
    sub a      ;Clear a.

    ld c,24

cleara:  ld b,64

clear_loop:
    out ($BE), a        ; Output tile number to the VDP data
    dec b
    jr nz, clear_loop
    dec c
    jr nz,cleara
    ret

Bomb:                 ;Here begins the actual game.
        ld a,(Ammo)   ;Check if there are any Ammos left?
        cp 0
        jp z,GameOver ;No...Game Over!
        ld a,(seed2)  ;Calculate pseudo random number.
        inc a
        ld (seed2),a
        ld l,a
        ld a,(seed1)
        add a,l
        adc a,a
        ld (seed1),a
        xor l
        ld e,l
        ld (seed2),a
        ld a,(prevr)
        ld l,a
        ld a,e
        sub l
        ld (prevr),a
        and 15  ;XPos=(rand()%8)*8+64 (16 random positions {at the center}).
        add a,a
        add a,a
        add a,a
        add a,64
        ld (XPosition),a    ;store it in XPosition
        sub a
        out ($BF), a        ; to the VDP control port.
        ld a, $7F
        out ($BF), a        ; Then we output this byte to the VDP
        ld a,183
        out ($BE),a
        ld (YPosition),a
        ld a,128
        out ($BF), a        ; to the VDP control port.
        ld a, $7F
        out ($BF), a        ; Then we output this byte to the VDP
        ld a,(XPosition)
        out ($BE),a
        ld a,213
        out ($BE),a
        sub a
        ld (DeltaX),a
        ld e,$7F

GameLoop:
     in a,($DC)
     bit 4,a                 ;Check if A button was pressed.
     jr z,Shot               ;Yes - exit from the loop.
     bit 5,a                 ;Check if B button was pressed.
     jr z,Shot
     jr GameLoop

Shot:
  ld hl, PSGMOD_SONG3
  call PSGMOD_INITIATE_NEW_SONG
  call PSGMOD_PLAY_NEXT_NOTE      ; play next note
     ld b,25                      ; Wait 2 VBLs
     rst $28
     call psg_init
     ld a,(Ammo)
     dec a
     ld (Ammo),a
     cp 10
     jp nc,Shoot
     ld a,24
     out ($BF),a

     ld a, $78
                
     out ($BF), a
                
     ld hl,ShootsText
     call PrintText
     ld a,(Ammo)
     add a,176
     out ($BE),a
Shoot:
VBL: in a,($7E)             ;Wait for vertical Blank.
     cp $B0
     jr nz, VBL
     in a,($DC)              ;Read controller.
     bit 2,a                 ;Check for Left.
     jr nz,NoLeft            ;No...
     ld a,-4
     ld (DeltaX),a
     jr NoNo

NoLeft:
     bit 3,a                 ;Check for Right.
     jr nz,NoNo              ;No...
     ld a,4
     ld (DeltaX),a
NoNo:
     ld a,(XPosition)
     ld l,a
     ld a,(DeltaX)
     cp -4
     jr nz,RightS
     ld a,l
     cp 0
     jr nz,RightS
     ld (DeltaX),a
RightS:
     ld a,(DeltaX)
     cp 4
     jr nz,YP1
     ld a,l
     cp 248
     jr nz,YP1
     sub a
     ld (DeltaX),a
YP1: ld a,(DeltaX)
     add a,l
     ld (XPosition),a
     ld b,a
     ld a,(YPosition)
     sub 4
     ld (YPosition),a
     ld l,a
     cp -13
     jp z,Bomb
     sub a
     out ($BF), a        ; to the VDP control port.
     ld a, e                    ; e=$7F for speed.
     out ($BF), a        ; Then we output this byte to the VDP
     ld a,l                     ; YPosition is in L
     out ($BE), a
     ld a,128
     out ($BF), a        ; to the VDP control port.
     ld a, e
     out ($BF), a        ; Then we output this byte to the VDP
     ld a,b                     ; XPosition is in B
     out ($BE), a
     add a,4
     srl a
     sra a    ;A=A/4 (XPosition = XPosition/4);
     ld d,a
     ld a,l
     srl a
     sra a
     sra a
     cp 0
     jr nz,d21
     inc a
d21: ld hl,Screen_address-$4000
     ld bc,64

Offsety: add hl,bc    ;Gets agents screen position, according to the level.
     dec a
     jr nz,Offsety
     ld c,d
     ld b,a
     add hl,bc
     ld a,l
     out ($BF), a        ; to the VDP control port.
     ld a, h
     out ($BF), a        ; Then we output this byte to the VDP
     in a,($BE)
     cp 159
     jr nc,Endof
     cp 149
     jr c,Endof
     call Collision

Endof:
     ld a, (XPosition)
     srl a
     sra a    ;A=A/4 (XPosition = XPosition/4);
     ld d,a
     ld a, (YPosition)
     add a,6
     srl a
     sra a
     sra a
     cp 0
     jr nz,d22
     inc a
d22: ld hl,Screen_address-$4000
     ld bc,64
Offsety2: add hl,bc    ;Gets agents screen position, according to the level.
     dec a
     jr nz,Offsety2
     ld c,d
     ld b,a
     add hl,bc
     ld a,l
     out ($BF), a        ; to the VDP control port.
     ld a, h
     out ($BF), a        ; Then we output this byte to the VDP
     in a,($BE)
     cp 159
     jr nc,Endof2
     cp 149
     jr c,Endof2
     call Collision

Endof2:
     ld a, (XPosition)
     add a,6
     srl a
     sra a    ;A=A/4 (XPosition = XPosition/4);
     ld d,a
     ld a, (YPosition)
     add a,6
     srl a
     sra a
     sra a
     cp 0
     jr nz,d23
     inc a
d23: ld hl,Screen_address-$4000
     ld bc,64
Offsety3: add hl,bc    ;Gets agents screen position, according to the level.
     dec a
     jr nz,Offsety3
     ld c,d
     ld b,a
     add hl,bc
     ld a,l
     out ($BF), a        ; to the VDP control port.
     ld a, h
     out ($BF), a        ; Then we output this byte to the VDP
     in a,($BE)
     cp 159
     jr nc,Endof3
     cp 149
     jr c,Endof3
     call Collision

Endof3: jp Shoot

NextStage:
     ld a,(Level)
     inc a
     ld (Level),a
     jp GLoop

GameOver:
     call vdp_clear
     rst $38
     ld bc,Screen_address+12
     ld a, c
     out ($BF), a
     ld a, b
     out ($BF), a
     ld hl,GameOverText
     call PrintText
     ld bc,Screen_address+15*64+22
     ld a, c
     out ($BF), a
     ld a, b
     out ($BF), a
     ld hl,GameOverText2
     call PrintText
   ld a,%00000100      ;Select second bank.
   ld ($FFFD),a
  ld hl, PSGMOD_SONG2
  call PSGMOD_INITIATE_NEW_SONG

     ld hl,palette3
     call SetPalette
     ld d,100
     ld c,50
GameOverScreen:
     ld b,2          ;2 vertical blanks.
     rst $28         ;Wait for vertical Blank.
  ld a,%00000100
  ld ($FFFD),a
  call PSGMOD_PLAY_NEXT_NOTE      ; play next note
  in a,($DC)
     bit 4,a                 ;Check if A button was pressed.
     jr z,GameOverEnd         ;Yes - exit from the loop.
     bit 5,a                 ;Check if B button was pressed.
     jr z,GameOverEnd
     dec d
     jr nz, GameOverScreen
     dec c
     jr nz, GameOverScreen
GameOverEnd:
     call psg_init
     ld b,55         ;55 vertical blanks.
     rst $28         ;Wait for vertical Blank.
     call vdp_clear
     ld hl,palette
     call SetPalette
     jp TitleScreen

Collision:
  push hl
  ld hl, PSGMOD_SONG
  call PSGMOD_INITIATE_NEW_SONG
  call PSGMOD_PLAY_NEXT_NOTE      ; play next note
     ld b,23                       ; Wait 24 VBLs
     rst $28
     call psg_init
     pop hl
     rst $38
     ld bc,$4000
     add hl,bc
     ld a,l
     out ($BF), a        ; to the VDP control port.
     ld a, h
     out ($BF), a        ; Then we output this byte to the VDP
     ld a,159
     out ($BE),a
     ld b,65
     rst $28
     ld a,l
     out ($BF), a        ; to the VDP control port.
     ld a, h
     out ($BF), a        ; Then we output this byte to the VDP
     sub a
     out ($BE),a
     ld a,(count)
     dec a
     ld (count),a
     cp 0
     pop hl             ;Correct the stack pointer.
     jp z,NextStage     ;Messes the stack, but I correct it.
     jp Bomb

.include "Mod1.ASM"
.include "PSGMOD.INC"
.BANK 1 SLOT 1
.ORG $0000
.include "260.ASM"
.include "317.ASM"
